import textwrap
import pytest

pytest_plugins = "pytester"


@pytest.fixture(name="emoji_tests")
def fixture_emoji_tests():
    return textwrap.dedent(
        """\
        import pytest

        def test_passed():
            assert "emoji" == "emoji"

        def test_failed():
            assert "emoji" == "hello world"

        @pytest.mark.xfail
        def test_xfailed():
            assert 1234 == 100

        @pytest.mark.xfail
        def test_xpassed():
            assert 1234 == 1234

        @pytest.mark.skipif(True, reason="don't run this test")
        def test_skipped():
            assert "emoji" == "emoji"

        @pytest.fixture
        def name():
            raise RuntimeError

        @pytest.mark.hello
        def test_error(name):
            assert name == "hello"
        """
    )


@pytest.fixture(name="custom_emojis")
def fixture_custom_emojis():
    return textwrap.dedent(
        """\
        def pytest_emoji_passed(config):
            return "🍪 ", "PASSED 🍪 "


        def pytest_emoji_failed(config):
            return "😿 ", "FAILED 😿 "


        def pytest_emoji_skipped(config):
            return "🙈 ", "SKIPPED 🙈 "


        def pytest_emoji_error(config):
            return "💩 ", "ERROR 💩 "


        def pytest_emoji_xfailed(config):
            return "🤓 ", "XFAIL 🤓 "


        def pytest_emoji_xpassed(config):
            return "😜 ", "XPASS 😜 "
        """
    )



def test_emoji_disabled_by_default_verbose(testdir, emoji_tests):
    # create a temporary pytest test module
    testdir.makepyfile(emoji_tests)

    # run pytest with the following cmd args
    result = testdir.runpytest("-v", "-o", "console_output_style=classic")

    # fnmatch_lines does an assertion internally
    result.stdout.fnmatch_lines(
        [
            "*::test_passed PASSED",
            "*::test_failed FAILED",
            "*::test_xfailed XFAIL",
            "*::test_xpassed XPASS",
            "*::test_skipped SKIPPED",
            "*::test_error ERROR",
        ]
    )

    # make sure that that we get a '1' exit code
    # as we have at least one failure
    assert result.ret == 1


def test_emoji_enabled_verbose(testdir, emoji_tests):
    # create a temporary pytest test module
    testdir.makepyfile(emoji_tests)

    # run pytest with the following cmd args
    result = testdir.runpytest(
        "-v", "--emoji", "-o", "console_output_style=classic"
    )

    # fnmatch_lines does an assertion internally
    result.stdout.fnmatch_lines(
        [
            "*::test_passed PASSED 😃 ",
            "*::test_failed FAILED 😰 ",
            "*::test_xfailed XFAIL 😞 ",
            "*::test_xpassed XPASS 😲 ",
            "*::test_skipped SKIPPED 🙄 ",
            "*::test_error ERROR 😡 ",
        ]
    )

    # make sure that that we get a '1' exit code
    # as we have at least one failure
    assert result.ret == 1


def test_emoji_enabled_custom_verbose(testdir, emoji_tests, custom_emojis):
    testdir.makeconftest(custom_emojis)

    # create a temporary pytest test module
    testdir.makepyfile(emoji_tests)

    # run pytest with the following cmd args
    result = testdir.runpytest(
        "-v", "--emoji", "-o", "console_output_style=classic"
    )

    # fnmatch_lines does an assertion internally
    result.stdout.fnmatch_lines(
        [
            "*::test_passed PASSED 🍪 ",
            "*::test_failed FAILED 😿 ",
            "*::test_xfailed XFAIL 🤓 ",
            "*::test_xpassed XPASS 😜 ",
            "*::test_skipped SKIPPED 🙈 ",
            "*::test_error ERROR 💩 ",
        ]
    )

    # make sure that that we get a '1' exit code
    # as we have at least one failure
    assert result.ret == 1


def test_emoji_disabled_by_default_non_verbose(testdir, emoji_tests):
    # create a temporary pytest test module
    testdir.makepyfile(emoji_tests)

    # run pytest with the following cmd args
    result = testdir.runpytest("-o", "console_output_style=classic")

    # fnmatch_lines does an assertion internally
    result.stdout.fnmatch_lines(["* .FxXsE"])

    # make sure that that we get a '1' exit code
    # as we have at least one failure
    assert result.ret == 1


def test_emoji_enabled_non_verbose(testdir, emoji_tests):
    # create a temporary pytest test module
    testdir.makepyfile(emoji_tests)

    # run pytest with the following cmd args
    result = testdir.runpytest("--emoji", "-o", "console_output_style=classic")

    # fnmatch_lines does an assertion internally
    result.stdout.fnmatch_lines(["* 😃 😰 😞 😲 🙄 😡 "])

    # make sure that that we get a '1' exit code
    # as we have at least one failure
    assert result.ret == 1


def test_emoji_enabled_custom_non_verbose(testdir, emoji_tests, custom_emojis):
    testdir.makeconftest(custom_emojis)

    # create a temporary pytest test module
    testdir.makepyfile(emoji_tests)

    # run pytest with the following cmd args
    result = testdir.runpytest("--emoji", "-o", "console_output_style=classic")

    # fnmatch_lines does an assertion internally
    result.stdout.fnmatch_lines(["* 🍪 😿 🤓 😜 🙈 💩 "])

    # make sure that that we get a '1' exit code
    # as we have at least one failure
    assert result.ret == 1



from pytest_emoji import hooks


def pytest_emoji_passed(config):
    # SMILING FACE WITH OPEN MOUTH
    return u"😃 ", u"PASSED 😃 "


def pytest_emoji_failed(config):
    # FACE WITH OPEN MOUTH AND COLD SWEAT
    return u"😰 ", u"FAILED 😰 "


def pytest_emoji_skipped(config):
    # FACE WITH ROLLING EYES
    return u"🙄 ", u"SKIPPED 🙄 "


def pytest_emoji_error(config):
    # POUTING FACE
    return u"😡 ", u"ERROR 😡 "


def pytest_emoji_xfailed(config):
    # DISAPPOINTED FACE
    return u"😞 ", u"XFAIL 😞 "


def pytest_emoji_xpassed(config):
    # DIZZY FACE
    return u"😲 ", u"XPASS 😲 "


def pytest_addhooks(pluginmanager):
    # Register new hooks from pytest_emoji.hooks
    pluginmanager.add_hookspecs(hooks)


def pytest_report_teststatus(report, config):
    if config.option.emoji is False:
        # Do not modify reporting, unless pytest is called with --emoji
        return

    # Handle error and skipped in setup and teardown phase
    if report.when in ("setup", "teardown"):
        if report.failed:
            short, verbose = config.hook.pytest_emoji_error(config=config)
            return "error", short, verbose
        elif report.skipped:
            short, verbose = config.hook.pytest_emoji_skipped(config=config)
            return "skipped", short, verbose

    # Handle xfailed and xpassed
    if hasattr(report, "wasxfail"):
        if report.skipped:
            short, verbose = config.hook.pytest_emoji_xfailed(config=config)
            return "xfailed", short, verbose
        elif report.passed:
            short, verbose = config.hook.pytest_emoji_xpassed(config=config)
            return "xpassed", short, verbose
        else:
            return "", "", ""

    # Handle passed, skipped and failed in call phase
    if report.when == "call":
        if report.passed:
            short, verbose = config.hook.pytest_emoji_passed(config=config)
        elif report.skipped:
            short, verbose = config.hook.pytest_emoji_skipped(config=config)
        elif report.failed:
            short, verbose = config.hook.pytest_emoji_failed(config=config)
        return report.outcome, short, verbose


def pytest_addoption(parser):
    group = parser.getgroup("emoji")
    group.addoption(
        "--emoji",
        action="store_true",
        default=False,
        help="Add emojis to the test result log.",
    )



import pathlib
import setuptools


def read(*args):
    file_path = pathlib.Path(__file__).parent.joinpath(*args)
    with file_path.open(encoding="utf-8") as f:
        return f.read()


setuptools.setup(
    name="pytest-emoji",
    version="0.2.0",
    author="Raphael Pierzina",
    author_email="raphael@hackebrot.de",
    maintainer="Raphael Pierzina",
    maintainer_email="raphael@hackebrot.de",
    license="MIT",
    url="https://github.com/hackebrot/pytest-emoji",
    description="A pytest plugin that adds emojis to your test result report",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages("src"),
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.4",
    install_requires=["pytest>=4.2.1"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: Pytest",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Testing",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
    ],
    entry_points={"pytest11": ["emoji = pytest_emoji.plugin"]},
    keywords=["pytest", "emoji"],
)
